#include <stdio.h>
#include <string.h>
#include <errno.h>

#include <sys/utsname.h>
#include <gtk/gtk.h>

#include "configmodel.h"

#define CONST_CPUINFO	"/proc/cpuinfo"

t_config_options read_configuration_options()
{
	t_config_options	retVal;
	struct utsname		uinfo;
	gchar			**strv;
	guint			len;
	gint			i;
	GError 			*err = NULL;
	gchar			*str = NULL;

	len		= 0;
	i		= 0;
	retVal.model	= NULL;
	retVal.cpu	= NULL;
	uname(&uinfo);

	retVal.model	= g_strdup("Unknown, not ARM");
	retVal.moduleType			= M_OTHER;
	retVal.wait_ack_before_sending_sms_pdu	= TRUE;
	retVal.at_cmds_in_frames		= FALSE;
	retVal.devName 				= "/dev/gsm0";
	retVal.initATCmd			= "";
	retVal.smsStoreATCmd			= "AT+CPMS=\"ME\",\"ME\",\"ME\"";

	/* get cpu info, only ARM for now */
	if (g_file_get_contents(CONST_CPUINFO, 
				&str,
				&len,
				&err))
	{
		strv		= g_strsplit(str,"\n",128);
		g_free(str);
		retVal.cpu	= g_strdup(strchr(strv[0],':') + 1);
		g_strstrip(retVal.cpu);
		while (strv[i])
		{
			if (strstr(strv[i], "Hardware"))
			{
				retVal.model	= g_strdup(strchr(strv[i],':')+1);
				g_strstrip(retVal.model);
				printf("hardware: %s", strv[i]);
				if ((strstr(strv[i], "HP iPAQ h6300")) ||	// kernel 2.6.16 and newer
				    (strstr(strv[i], "HP iPAQ H6300")))		// kernel older than 2.6.16
				{
					g_free(retVal.model);
					retVal.model				= g_strdup("HP iPAQ h6300");
					retVal.moduleType			= M_WAVECOM;
					retVal.wait_ack_before_sending_sms_pdu	= FALSE;
					retVal.at_cmds_in_frames		= FALSE;
					retVal.devName 				= "/dev/ttyS1";
					/*
					 * 1) Turn off the multiplexing by using AT+WMUX=0
					 * 2) Turn the responses to verpose mode by using command ATV1
					 * --> combination of those = AT+WMUX=0;V1
					 */
					retVal.initATCmd			= "AT+WMUX=0;V1";
					retVal.smsStoreATCmd			= "AT+CPMS=\"ME\",\"ME\",\"ME\"";
				}
				else if (strstr(strv[i],"HTC Blueangel"))
				{
					g_free(retVal.model);
					retVal.model	= g_strdup("HTC Blueangel");
					retVal.moduleType			= M_OTHER;
					retVal.wait_ack_before_sending_sms_pdu	= TRUE;
					retVal.at_cmds_in_frames		= TRUE;
					retVal.devName 				= "/dev/ttyS0";
					retVal.initATCmd			= "";
					retVal.smsStoreATCmd			= "AT+CPMS=\"SM\",\"SM\",\"SM\"";											
				}
				else if (strstr(strv[i],"HTC Magician"))
				{
					g_free(retVal.model);
					retVal.model	= g_strdup("HTC Magician");
					retVal.moduleType			= M_OTHER;
					retVal.wait_ack_before_sending_sms_pdu	= TRUE;
					retVal.at_cmds_in_frames		= TRUE;
					retVal.devName 				= "/dev/ttyS0";
					retVal.initATCmd			= "";
					retVal.smsStoreATCmd			= "AT+CPMS=\"SM\",\"SM\",\"SM\"";											
				}
				else if (strstr(strv[i],"HTC Universal"))
				{
					g_free(retVal.model);
					retVal.model	= g_strdup("HTC Universal");
					retVal.moduleType			= M_OTHER;
					retVal.wait_ack_before_sending_sms_pdu	= TRUE;
					retVal.at_cmds_in_frames		= TRUE;
					retVal.devName 				= "/dev/ttyS0";
					retVal.initATCmd			= "";
					retVal.smsStoreATCmd			= "AT+CPMS=\"SM\",\"SM\",\"SM\"";											
				}
				else if (strstr(strv[i],"HTC Himalaya"))
				{
					g_free(retVal.model);
					retVal.model	= g_strdup("HTC Himalaya");
					retVal.moduleType			= M_HIMALAYA;
					retVal.wait_ack_before_sending_sms_pdu	= TRUE;
					retVal.at_cmds_in_frames		= TRUE;
					retVal.devName 				= "/dev/ttyS2";
					retVal.initATCmd			= "";
					retVal.smsStoreATCmd			= "AT+CPMS=\"SM\",\"SM\",\"SM\"";											
				}
				else
				{
					g_free(retVal.model);
					retVal.model	= g_strdup("Unknown");
				}
				break;
			}
			i++;
		}
		g_strfreev(strv);
	}
	return retVal;
}
