#include <glib.h>
#include <alsa/asoundlib.h>
#include <gpe/errorbox.h>

#ifndef DEFAULT_CARD
#define DEFAULT_CARD "default"
#endif


static snd_mixer_t *handle;
static long phone_vol_min, phone_vol_max;
static snd_mixer_elem_t *elem_phone, 
			*elem_mic_pwr,
			// elem_playback is h6300 specific
			*elem_playback;
 
static int		old_playback_item;

int mixer_open(void)
{
	snd_mixer_selem_id_t *sid;
	if (snd_mixer_open(&handle, 0) < 0) {
		gpe_error_box("can't open mixer");
		goto err0;
	}
	if (snd_mixer_attach(handle, DEFAULT_CARD) <0) {
		gpe_error_box("can't attach default card mixer");
		goto err1;
	}
	if (snd_mixer_selem_register(handle, NULL, NULL) < 0) {
		gpe_error_box("can't get mixer elements");
		goto err2;
	}
	if (snd_mixer_load(handle) < 0) {
		gpe_error_box("can't load mixer");
		goto err2;
	}
	snd_mixer_selem_id_alloca(&sid);
	
	snd_mixer_selem_id_set_name(sid, "Target");
	elem_playback = snd_mixer_find_selem(handle, sid);
	if (elem_playback) {
		if (snd_mixer_selem_is_enumerated(elem_playback)) {
			/* h6300 specific way where "cellphone" target can be selected 
			 * from the mixer router element
			 */
			snd_mixer_selem_get_enum_item(elem_playback, 0, &old_playback_item);
			printf("index fo the selected Target item = %d\n", old_playback_item);
		}
		else {
			printf("Target is not enumerated switch\n");
			elem_playback	= NULL;
		}
	}
	if (elem_playback == NULL) {
		/* non-ipaq h6300 specific way*/
		snd_mixer_selem_id_set_name(sid, "Phone");
		elem_phone = snd_mixer_find_selem(handle, sid);
		if (!elem_phone) {
			gpe_error_box("can't get mixer element 'Phone'");
			goto err2;
		}
		if (!snd_mixer_selem_has_playback_volume(elem_phone)) {
			gpe_error_box("mixer element 'Phone' of wrong type");
			goto err2;
		}
		snd_mixer_selem_get_playback_volume_range(elem_phone, &phone_vol_min, &phone_vol_max);
		snd_mixer_selem_id_set_name(sid, "Mic Power");
		elem_mic_pwr = snd_mixer_find_selem (handle, sid);
		if (!elem_mic_pwr) {
			gpe_error_box("can't get mixer element 'Mic Power'");
			goto err2;
		}
		if (!snd_mixer_selem_has_playback_switch(elem_mic_pwr)) {
			gpe_error_box("mixer element 'Mic Power' of wrong type");
			goto err2;
		}
	}
	return TRUE;
err2:
	snd_mixer_detach(handle, DEFAULT_CARD);
err1:
	snd_mixer_free(handle);	
err0:
	elem_phone	= NULL;
	elem_mic_pwr	= NULL;
	elem_playback	= NULL;
	return FALSE;
}

void mixer_close(void)
{
	snd_mixer_detach(handle, DEFAULT_CARD);
	snd_mixer_free(handle);	
}

void mixer_phone_volume_set(int val)
{
	float volume;
	int tmp;

	if (!elem_phone)
		return;

	volume = (float) val / 100 * (phone_vol_max - phone_vol_min);
	volume = CLAMP (volume, phone_vol_min, phone_vol_max);
	tmp = volume+0.5;

	snd_mixer_selem_set_playback_volume_all(elem_phone, tmp);
}

void mixer_mic_power_set(int val)
{
	if (!elem_mic_pwr)
		return;

	snd_mixer_selem_set_playback_switch_all(elem_mic_pwr, val > 0);
}

/**
 * If mixer has Playback route mixer element, try to select "Cellphone" from it as a playback target.
 * This method is executed when phone call is established.
 */
void select_phone_as_playback_target()
{
	if (elem_playback) {
		// get the old playback item to safe
		snd_mixer_selem_get_enum_item(elem_playback, 0, &old_playback_item);
		printf("select_phone_as_playback_target(), old_playback_target = %d\n", old_playback_item);
		// change the playback item to "Cellphone"
		snd_mixer_selem_set_enum_item(elem_playback, 0, 2);
	}
	else {
		printf("select_phone_as_playback_target(), elem_playback not available\n");
	}
}

void restore_original_playback_target()
{
	if (elem_playback) {
		// change the playback item to "Cellphone"
		printf("restore_original_playback_target(), restored_target = %d\n", old_playback_item);
		snd_mixer_selem_set_enum_item(elem_playback, 0, old_playback_item);
	}
	else {
		printf("restore_original_playback_target(), elem_playback not available\n");
	}	
}
